@@ -6,8 +6,8 @@
 
    This file is part of LibTMCG.
 
- Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007 
-               Heiko Stamer <stamer@gaos.org>
+ Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2015, 2016, 2017 
+               Heiko Stamer <HeikoStamer@gmx.net>
 
    LibTMCG is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -24,6 +24,10 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/
 
+// include headers
+#ifdef HAVE_CONFIG_H
+	#include "libTMCG_config.h"
+#endif
 #include "SchindelhauerTMCG.hh"
 
 SchindelhauerTMCG::SchindelhauerTMCG
@@ -32,6 +36,7 @@ SchindelhauerTMCG::SchindelhauerTMCG
 {
 	assert(TMCG_Players <= TMCG_MAX_PLAYERS);
 	assert(TMCG_TypeBits <= TMCG_MAX_TYPEBITS);
+	assert(TMCG_SecurityLevel <= TMCG_MAX_ZNP_ITERATIONS);
 	
 	TMCG_MaxCardType = 1;
 	for (size_t i = 0; i < TMCG_TypeBits; i++)
@@ -60,7 +65,7 @@ void SchindelhauerTMCG::TMCG_ProveQuadraticResidue
 		key.gcdext_up, key.gcdext_vq, key.pa1d4, key.qa1d4);
 	
 	// phase (P2)
-	for (unsigned long int i = 0; i < security_desire; i++)
+	for (unsigned long int i = 0; (i < security_desire) && (i < TMCG_MAX_ZNP_ITERATIONS); i++)
 	{
 		mpz_ptr r = new mpz_t(), s = new mpz_t();
 		mpz_init(r), mpz_init(s);
@@ -74,7 +79,7 @@ void SchindelhauerTMCG::TMCG_ProveQuadraticResidue
 		while (mpz_cmp_ui(lej, 1L) || !mpz_cmp_ui(r, 1L));
 		
 		// compute $s := t_sqrt \cdot r_i^{-1} \bmod m$
-		ret = mpz_invert(s, r, key.m);
+		int ret = mpz_invert(s, r, key.m);
 		assert(ret);
 		mpz_mul(s, s, t_sqrt);
 		mpz_mod(s, s, key.m);
@@ -99,7 +104,7 @@ void SchindelhauerTMCG::TMCG_ProveQuadraticResidue
 	}
 	
 	// phase (P4)
-	for (unsigned long int i = 0; i < security_desire; i++)
+	for (unsigned long int i = 0; (i < security_desire) && (i < TMCG_MAX_ZNP_ITERATIONS); i++)
 	{
 		// receive R/S-question from the verifier
 		in >> foo;
@@ -113,9 +118,9 @@ void SchindelhauerTMCG::TMCG_ProveQuadraticResidue
 	
 	mpz_clear(foo), mpz_clear(bar), mpz_clear(lej), mpz_clear(t_sqrt);
 	for (std::vector<mpz_ptr>::iterator ri = rr.begin(); ri != rr.end(); ri++)
-		mpz_clear(*ri), delete *ri;
+		mpz_clear(*ri), delete [] *ri;
 	for (std::vector<mpz_ptr>::iterator si = ss.begin(); si != ss.end(); si++)
-		mpz_clear(*si), delete *si;
+		mpz_clear(*si), delete [] *si;
 }
 
 bool SchindelhauerTMCG::TMCG_VerifyQuadraticResidue
@@ -177,9 +182,9 @@ bool SchindelhauerTMCG::TMCG_VerifyQuadraticResidue
 	{
 		mpz_clear(foo), mpz_clear(bar), mpz_clear(lej);
 		for (std::vector<mpz_ptr>::iterator ri = RR.begin(); ri != RR.end(); ri++)
-			mpz_clear(*ri), delete *ri;
+			mpz_clear(*ri), delete [] *ri;
 		for (std::vector<mpz_ptr>::iterator si = SS.begin(); si != SS.end(); si++)
-			mpz_clear(*si), delete *si;
+			mpz_clear(*si), delete [] *si;
 		return return_value;
 	}
 }
@@ -289,7 +294,7 @@ void SchindelhauerTMCG::TMCG_ProveMaskValue
 	try
 	{
 		// phase (P2)
-		for (unsigned long int i = 0; i < security_desire; i++)
+		for (unsigned long int i = 0; (i < security_desire) && (i < TMCG_MAX_ZNP_ITERATIONS); i++)
 		{
 			mpz_ptr r2 = new mpz_t(), b2 = new mpz_t();
 			mpz_init(r2), mpz_init(b2);
@@ -326,7 +331,7 @@ void SchindelhauerTMCG::TMCG_ProveMaskValue
 		}
 		
 		// phase (P4)
-		for (unsigned long int i = 0; i < security_desire; i++)
+		for (unsigned long int i = 0; (i < security_desire) && (i < TMCG_MAX_ZNP_ITERATIONS); i++)
 		{
 			// receive Z/Z'-question from verifier
 			in >> foo;
@@ -365,9 +370,9 @@ void SchindelhauerTMCG::TMCG_ProveMaskValue
 	{
 		mpz_clear(foo), mpz_clear(bar);
 		for (std::vector<mpz_ptr>::iterator ri = rr.begin(); ri != rr.end(); ri++)
-			mpz_clear(*ri), delete *ri;
+			mpz_clear(*ri), delete [] *ri;
 		for (std::vector<mpz_ptr>::iterator bi = bb.begin(); bi != bb.end(); bi++)
-			mpz_clear(*bi), delete *bi;
+			mpz_clear(*bi), delete [] *bi;
 		return;
 	}
 }
@@ -422,7 +427,7 @@ bool SchindelhauerTMCG::TMCG_VerifyMaskValue
 	{
 		mpz_clear(foo), mpz_clear(bar), mpz_clear(lej);
 		for (std::vector<mpz_ptr>::iterator ti = T.begin(); ti != T.end(); ti++)
-			mpz_clear(*ti), delete *ti;
+			mpz_clear(*ti), delete [] *ti;
 		return return_value;
 	}
 }
@@ -438,13 +443,13 @@ void SchindelhauerTMCG::TMCG_ProveMaskOne
 	
 	// compute y1m = y^{-1} mod m
 	mpz_init(y1m);
-	ret = mpz_invert(y1m, key.y, key.m);
+	int ret = mpz_invert(y1m, key.y, key.m);
 	assert(ret);
 	
 	mpz_init(foo), mpz_init(bar), mpz_init(tim);
 	
 	// phase (P2)
-	for (unsigned long int i = 0; i < security_desire; i++)
+	for (unsigned long int i = 0; (i < security_desire) && (i < TMCG_MAX_ZNP_ITERATIONS); i++)
 	{
 		mpz_ptr r3 = new mpz_t(), s = new mpz_t(),
 			b3 = new mpz_t(), c = new mpz_t();
@@ -542,7 +547,7 @@ void SchindelhauerTMCG::TMCG_ProveMaskOne
 	}
 	
 	// phase (P4)
-	for (unsigned long int i = 0; i < security_desire; i++)
+	for (unsigned long int i = 0; (i < security_desire) && (i < TMCG_MAX_ZNP_ITERATIONS); i++)
 	{
 		// receive R/S-question from verifier
 		in >> foo;
@@ -556,13 +561,13 @@ void SchindelhauerTMCG::TMCG_ProveMaskOne
 	
 	mpz_clear(y1m), mpz_clear(foo), mpz_clear(bar), mpz_clear(tim);
 	for (std::vector<mpz_ptr>::iterator ri = rr.begin(); ri != rr.end(); ri++)
-		mpz_clear(*ri), delete *ri;
+		mpz_clear(*ri), delete [] *ri;
 	for (std::vector<mpz_ptr>::iterator bi = bb.begin(); bi != bb.end(); bi++)
-		mpz_clear(*bi), delete *bi;
+		mpz_clear(*bi), delete [] *bi;
 	for (std::vector<mpz_ptr>::iterator si = ss.begin(); si != ss.end(); si++)
-		mpz_clear(*si), delete *si;
+		mpz_clear(*si), delete [] *si;
 	for (std::vector<mpz_ptr>::iterator ci = cc.begin(); ci != cc.end(); ci++)
-		mpz_clear(*ci), delete *ci;
+		mpz_clear(*ci), delete [] *ci;
 }
 
 bool SchindelhauerTMCG::TMCG_VerifyMaskOne
@@ -626,9 +631,9 @@ bool SchindelhauerTMCG::TMCG_VerifyMaskOne
 	{	
 		mpz_clear(foo), mpz_clear(bar), mpz_clear(lej);
 		for (std::vector<mpz_ptr>::iterator ri = RR.begin(); ri != RR.end(); ri++)
-			mpz_clear(*ri), delete *ri;
+			mpz_clear(*ri), delete [] *ri;
 		for (std::vector<mpz_ptr>::iterator si = SS.begin(); si != SS.end(); si++)
-			mpz_clear(*si), delete *si;
+			mpz_clear(*si), delete [] *si;
 		return return_value;
 	}
 }
@@ -645,7 +650,7 @@ void SchindelhauerTMCG::TMCG_ProveNonQuadraticResidue_PerfectZeroKnowledge
 	try
 	{
 		// phase (P2) and (P3)
-		for (unsigned long int i = 0; i < security_desire; i++)
+		for (unsigned long int i = 0; (i < security_desire) && (i < TMCG_MAX_ZNP_ITERATIONS); i++)
 		{
 			// receive question
 			in >> foo;
@@ -1135,12 +1140,26 @@ void random_permutation_fast
 	
 	for (size_t i = 0; i < (n - 1); i++)
 	{
-		size_t tmp = pi[i], rnd = i + mpz_srandom_mod(n - i);
+		size_t tmp = pi[i], rnd = i + (size_t)mpz_srandom_mod(n - i);
 		pi[i] = pi[rnd];
 		pi[rnd] = tmp;
 	}
 }
 
+// create a random rotation (naive algorithm)
+size_t random_rotation
+	(size_t n, std::vector<size_t> &pi)
+{
+	size_t r = (size_t)mpz_srandom_mod(n);
+	pi.clear();
+	for (size_t i = 0; i < n; i++)
+		pi.push_back((r + i) % n);
+	if (n > 0)
+		return ((n - r) % n);
+	else
+		return 0;
+}
+
 size_t SchindelhauerTMCG::TMCG_CreateStackSecret
 	(TMCG_StackSecret<TMCG_CardSecret> &ss, bool cyclic,
 		const TMCG_PublicKeyRing &ring, size_t index, size_t size)
@@ -1149,12 +1168,12 @@ size_t SchindelhauerTMCG::TMCG_CreateStackSecret
 	assert(ring.keys.size() > index);
 	assert(size <= TMCG_MAX_CARDS);
 	
-	size_t cyc = 0;
+	size_t r = 0;
 	std::vector<size_t> pi;
 	
 	ss.clear();
 	if (cyclic)
-		cyc = (size_t)mpz_srandom_mod(size);
+		r = random_rotation(size, pi); // naive alorithm
 	else
 		random_permutation_fast(size, pi); // Knuth's algorithm
 
@@ -1163,9 +1182,9 @@ size_t SchindelhauerTMCG::TMCG_CreateStackSecret
 		TMCG_CardSecret cs(TMCG_Players, TMCG_TypeBits);
 		TMCG_CreateCardSecret(cs, ring, index);
 		
-		ss.push(((cyclic) ? (cyc + i) % size : pi[i]), cs);
+		ss.push(pi[i], cs);
 	}
-	return cyc;
+	return r;
 }
 
 size_t SchindelhauerTMCG::TMCG_CreateStackSecret
@@ -1174,12 +1193,12 @@ size_t SchindelhauerTMCG::TMCG_CreateStackSecret
 {
 	assert(size <= TMCG_MAX_CARDS);
 	
-	size_t cyc = 0;
+	size_t r = 0;
 	std::vector<size_t> pi;
 	
 	ss.clear();
 	if (cyclic)
-		cyc = (size_t)mpz_srandom_mod(size);
+		r = random_rotation(size, pi); // naive alorithm
 	else
 		random_permutation_fast(size, pi); // Knuth's algorithm
 	
@@ -1188,9 +1207,41 @@ size_t SchindelhauerTMCG::TMCG_CreateStackSecret
 		VTMF_CardSecret cs;
 		TMCG_CreateCardSecret(cs, vtmf);
 		
-		ss.push(((cyclic) ? (cyc + i) % size : pi[i]), cs);
+		ss.push(pi[i], cs);
+	}
+	return r;
+}
+
+void SchindelhauerTMCG::TMCG_CreateStackSecret
+	(TMCG_StackSecret<TMCG_CardSecret> &ss, const std::vector<size_t> &pi,
+		const TMCG_PublicKeyRing &ring, size_t index, size_t size)
+{
+	assert(ring.keys.size() == TMCG_Players);
+	assert(ring.keys.size() > index);
+	assert(size <= TMCG_MAX_CARDS);
+	
+	for (size_t i = 0; i < size; i++)
+	{
+		TMCG_CardSecret cs(TMCG_Players, TMCG_TypeBits);
+		TMCG_CreateCardSecret(cs, ring, index);
+		
+		ss.push(pi[i], cs);
+	}
+}
+
+void SchindelhauerTMCG::TMCG_CreateStackSecret
+	(TMCG_StackSecret<VTMF_CardSecret> &ss, const std::vector<size_t> &pi,
+	size_t size, BarnettSmartVTMF_dlog *vtmf)
+{
+	assert(size <= TMCG_MAX_CARDS);
+		
+	for (size_t i = 0; i < size; i++)
+	{
+		VTMF_CardSecret cs;
+		TMCG_CreateCardSecret(cs, vtmf);
+		
+		ss.push(pi[i], cs);
 	}
-	return cyc;
 }
 
 void SchindelhauerTMCG::TMCG_MixStack
@@ -1331,7 +1382,7 @@ void SchindelhauerTMCG::TMCG_ProveStackEquality
 	in >> security_desire, in.ignore(1, '\n');
 	
 	mpz_init(foo);
-	for (unsigned long int i = 0; i < security_desire; i++)
+	for (unsigned long int i = 0; (i < security_desire) && (i < TMCG_MAX_ZNP_ITERATIONS); i++)
 	{
 		TMCG_Stack<TMCG_Card> s3;
 		TMCG_StackSecret<TMCG_CardSecret> ss2;
@@ -1377,7 +1428,7 @@ void SchindelhauerTMCG::TMCG_ProveStackEquality
 	in >> security_desire, in.ignore(1, '\n');
 	
 	mpz_init(foo);
-	for (unsigned long int i = 0; i < security_desire; i++)
+	for (unsigned long int i = 0; (i < security_desire) && (i < TMCG_MAX_ZNP_ITERATIONS); i++)
 	{
 		TMCG_Stack<VTMF_Card> s3;
 		TMCG_StackSecret<VTMF_CardSecret> ss2;
@@ -1448,6 +1499,26 @@ void SchindelhauerTMCG::TMCG_InitializeStackEquality_Groth
 	}
 }
 
+void SchindelhauerTMCG::TMCG_InitializeStackEquality_Hoogh
+	(std::vector<mpz_ptr> &R,
+	std::vector<std::pair<mpz_ptr, mpz_ptr> > &e,
+	std::vector<std::pair<mpz_ptr, mpz_ptr> > &E,
+	const TMCG_Stack<VTMF_Card> &s, const TMCG_Stack<VTMF_Card> &s2,
+	const TMCG_StackSecret<VTMF_CardSecret> &ss)
+{
+	for (size_t i = 0; i < s.size(); i++)
+	{
+		mpz_ptr tmp = new mpz_t(), tmp4 = new mpz_t(), tmp5 = new mpz_t(),
+			tmp6 = new mpz_t(), tmp7 = new mpz_t();
+		mpz_init_set(tmp, ss[ss[i].first].second.r),
+			mpz_init_set(tmp4, s[i].c_1), mpz_init_set(tmp5, s[i].c_2),
+			mpz_init_set(tmp6, s2[i].c_1), mpz_init_set(tmp7, s2[i].c_2);
+		R.push_back(tmp),
+			e.push_back(std::pair<mpz_ptr, mpz_ptr>(tmp4, tmp5)),
+			E.push_back(std::pair<mpz_ptr, mpz_ptr>(tmp6, tmp7));
+	}
+}
+
 void SchindelhauerTMCG::TMCG_ReleaseStackEquality_Groth
 	(std::vector<size_t> &pi, std::vector<mpz_ptr> &R,
 	std::vector<std::pair<mpz_ptr, mpz_ptr> > &e,
@@ -1455,15 +1526,31 @@ void SchindelhauerTMCG::TMCG_ReleaseStackEquality_Groth
 {
 	for (size_t i = 0; i < pi.size(); i++)
 	{
-		mpz_clear(R[i]), delete R[i];
+		mpz_clear(R[i]), delete [] R[i];
 		mpz_clear(e[i].first), mpz_clear(e[i].second);
-		delete e[i].first, delete e[i].second;
+		delete [] e[i].first, delete [] e[i].second;
 		mpz_clear(E[i].first), mpz_clear(E[i].second);
-		delete E[i].first, delete E[i].second;
+		delete [] E[i].first, delete [] E[i].second;
 	}
 	pi.clear(), R.clear(), e.clear(), E.clear();
 }
 
+void SchindelhauerTMCG::TMCG_ReleaseStackEquality_Hoogh
+	(std::vector<mpz_ptr> &R,
+	std::vector<std::pair<mpz_ptr, mpz_ptr> > &e,
+	std::vector<std::pair<mpz_ptr, mpz_ptr> > &E)
+{
+	for (size_t i = 0; i < R.size(); i++)
+	{
+		mpz_clear(R[i]), delete [] R[i];
+		mpz_clear(e[i].first), mpz_clear(e[i].second);
+		delete [] e[i].first, delete [] e[i].second;
+		mpz_clear(E[i].first), mpz_clear(E[i].second);
+		delete [] E[i].first, delete [] E[i].second;
+	}
+	R.clear(), e.clear(), E.clear();
+}
+
 void SchindelhauerTMCG::TMCG_ReleaseStackEquality_Groth
 	(std::vector<std::pair<mpz_ptr, mpz_ptr> > &e,
 	std::vector<std::pair<mpz_ptr, mpz_ptr> > &E)
@@ -1471,9 +1558,9 @@ void SchindelhauerTMCG::TMCG_ReleaseStackEquality_Groth
 	for (size_t i = 0; i < e.size(); i++)
 	{
 		mpz_clear(e[i].first), mpz_clear(e[i].second);
-		delete e[i].first, delete e[i].second;
+		delete [] e[i].first, delete [] e[i].second;
 		mpz_clear(E[i].first), mpz_clear(E[i].second);
-		delete E[i].first, delete E[i].second;
+		delete [] E[i].first, delete [] E[i].second;
 	}
 	e.clear(), E.clear();
 }
@@ -1496,10 +1583,86 @@ void SchindelhauerTMCG::TMCG_ProveStackEquality_Groth
 	std::vector<mpz_ptr> R;
 	std::vector<std::pair<mpz_ptr, mpz_ptr> > e, E;
 	std::vector<size_t> pi;
+
+	JareckiLysyanskayaEDCF *edcf = new JareckiLysyanskayaEDCF(2, 0,
+		vtmf->p, vtmf->q, vtmf->g, vtmf->h);
 	
 	TMCG_InitializeStackEquality_Groth(pi, R, e, E, s, s2, ss);
-	vsshe->Prove_interactive(pi, R, e, E, in, out);
+	vsshe->Prove_interactive_publiccoin(pi, R, e, E, edcf, in, out);
 	TMCG_ReleaseStackEquality_Groth(pi, R, e, E);
+
+	delete edcf;
+}
+
+void SchindelhauerTMCG::TMCG_ProveStackEquality_Groth_noninteractive
+	(const TMCG_Stack<VTMF_Card> &s, const TMCG_Stack<VTMF_Card> &s2,
+	const TMCG_StackSecret<VTMF_CardSecret> &ss,
+	BarnettSmartVTMF_dlog *vtmf, GrothVSSHE *vsshe,
+	std::ostream &out)
+{
+	assert((s.size() == s2.size()) && (s.size() == ss.size()));
+	assert(!mpz_cmp(vtmf->h, vsshe->com->h));
+	assert(!mpz_cmp(vtmf->q, vsshe->com->q));
+	assert(!mpz_cmp(vtmf->p, vsshe->p));
+	assert(!mpz_cmp(vtmf->q, vsshe->q));
+	assert(!mpz_cmp(vtmf->g, vsshe->g));
+	assert(!mpz_cmp(vtmf->h, vsshe->h));
+	assert((s.size() <= vsshe->com->g.size()));
+	
+	std::vector<mpz_ptr> R;
+	std::vector<std::pair<mpz_ptr, mpz_ptr> > e, E;
+	std::vector<size_t> pi;
+	
+	TMCG_InitializeStackEquality_Groth(pi, R, e, E, s, s2, ss);
+	vsshe->Prove_noninteractive(pi, R, e, E, out);
+	TMCG_ReleaseStackEquality_Groth(pi, R, e, E);
+}
+
+void SchindelhauerTMCG::TMCG_ProveStackEquality_Hoogh
+	(const TMCG_Stack<VTMF_Card> &s, const TMCG_Stack<VTMF_Card> &s2,
+	const TMCG_StackSecret<VTMF_CardSecret> &ss,
+	BarnettSmartVTMF_dlog *vtmf, HooghSchoenmakersSkoricVillegasVRHE *vrhe,
+	std::istream &in, std::ostream &out)
+{
+	assert((s.size() == s2.size()) && (s.size() == ss.size()));
+	assert(!mpz_cmp(vtmf->p, vrhe->p));
+	assert(!mpz_cmp(vtmf->q, vrhe->q));
+	assert(!mpz_cmp(vtmf->g, vrhe->g));
+	assert(!mpz_cmp(vtmf->h, vrhe->h));
+	
+	std::vector<mpz_ptr> R;
+	std::vector<std::pair<mpz_ptr, mpz_ptr> > e, E;
+	size_t r = (ss.size() - ss[0].first) % ss.size();
+
+	JareckiLysyanskayaEDCF *edcf = new JareckiLysyanskayaEDCF(2, 0,
+		vtmf->p, vtmf->q, vtmf->g, vtmf->h);
+	
+	TMCG_InitializeStackEquality_Hoogh(R, e, E, s, s2, ss);
+	vrhe->Prove_interactive_publiccoin(r, R, e, E, edcf, in, out);
+	TMCG_ReleaseStackEquality_Hoogh(R, e, E);
+
+	delete edcf;
+}
+
+void SchindelhauerTMCG::TMCG_ProveStackEquality_Hoogh_noninteractive
+	(const TMCG_Stack<VTMF_Card> &s, const TMCG_Stack<VTMF_Card> &s2,
+	const TMCG_StackSecret<VTMF_CardSecret> &ss,
+	BarnettSmartVTMF_dlog *vtmf, HooghSchoenmakersSkoricVillegasVRHE *vrhe,
+	std::ostream &out)
+{
+	assert((s.size() == s2.size()) && (s.size() == ss.size()));
+	assert(!mpz_cmp(vtmf->p, vrhe->p));
+	assert(!mpz_cmp(vtmf->q, vrhe->q));
+	assert(!mpz_cmp(vtmf->g, vrhe->g));
+	assert(!mpz_cmp(vtmf->h, vrhe->h));
+	
+	std::vector<mpz_ptr> R;
+	std::vector<std::pair<mpz_ptr, mpz_ptr> > e, E;
+	size_t r = (ss.size() - ss[0].first) % ss.size();
+	
+	TMCG_InitializeStackEquality_Hoogh(R, e, E, s, s2, ss);
+	vrhe->Prove_noninteractive(r, R, e, E, out);
+	TMCG_ReleaseStackEquality_Hoogh(R, e, E);
 }
 
 bool SchindelhauerTMCG::TMCG_VerifyStackEquality
@@ -1565,9 +1728,9 @@ bool SchindelhauerTMCG::TMCG_VerifyStackEquality
 			// verify cyclic shift
 			if (cyclic)
 			{
-				size_t cy = ss[0].first;
+				size_t cyc = ss[0].first;
 				for (size_t j = 1; j < ss.size(); j++)
-					if (((++cy) % ss.size()) != ss[j].first)
+					if (((++cyc) % ss.size()) != ss[j].first)
 						throw false;
 			}
 		}
@@ -1693,11 +1856,112 @@ bool SchindelhauerTMCG::TMCG_VerifyStackEquality_Groth
 	
 	std::vector<mpz_ptr> R;
 	std::vector<std::pair<mpz_ptr, mpz_ptr> > e, E;
-	std::vector<size_t> pi;
+
+	JareckiLysyanskayaEDCF *edcf = new JareckiLysyanskayaEDCF(2, 0,
+		vtmf->p, vtmf->q, vtmf->g, vtmf->h);
 	
 	TMCG_InitializeStackEquality_Groth(e, E, s, s2);
-	bool return_value = vsshe->Verify_interactive(e, E, in, out);
+	bool return_value = vsshe->Verify_interactive_publiccoin(e, E, edcf, in, out);
 	TMCG_ReleaseStackEquality_Groth(e, E);
+
+	delete edcf;
+	
+	return return_value;
+}
+
+bool SchindelhauerTMCG::TMCG_VerifyStackEquality_Groth_noninteractive
+	(const TMCG_Stack<VTMF_Card> &s, const TMCG_Stack<VTMF_Card> &s2,
+	BarnettSmartVTMF_dlog *vtmf, GrothVSSHE *vsshe,
+	std::istream &in)
+{
+	// check whether the parameters of VSSHE and VTMF match
+	if (mpz_cmp(vtmf->h, vsshe->com->h) || mpz_cmp(vtmf->q, vsshe->com->q) || 
+		mpz_cmp(vtmf->p, vsshe->p) || mpz_cmp(vtmf->q, vsshe->q) || 
+		mpz_cmp(vtmf->g, vsshe->g) || mpz_cmp(vtmf->h, vsshe->h) || 
+		(s.size() > vsshe->com->g.size()))
+			return false;
+	
+	if (s.size() != s2.size())
+		return false;
+	
+	// check whether the elements of the shuffled stack belong to the group
+	for (size_t i = 0; i < s2.size(); i++)
+	{
+		if (!vtmf->CheckElement(s2[i].c_1) || !vtmf->CheckElement(s2[i].c_2))
+			return false;
+	}
+	
+	std::vector<mpz_ptr> R;
+	std::vector<std::pair<mpz_ptr, mpz_ptr> > e, E;
+	
+	TMCG_InitializeStackEquality_Groth(e, E, s, s2);
+	bool return_value = vsshe->Verify_noninteractive(e, E, in);
+	TMCG_ReleaseStackEquality_Groth(e, E);
+	
+	return return_value;
+}
+
+bool SchindelhauerTMCG::TMCG_VerifyStackEquality_Hoogh
+	(const TMCG_Stack<VTMF_Card> &s, const TMCG_Stack<VTMF_Card> &s2,
+	BarnettSmartVTMF_dlog *vtmf, HooghSchoenmakersSkoricVillegasVRHE *vrhe,
+	std::istream &in, std::ostream &out)
+{
+	// check whether the parameters of VRHE and VTMF match
+	if (mpz_cmp(vtmf->p, vrhe->p) || mpz_cmp(vtmf->q, vrhe->q) || 
+		mpz_cmp(vtmf->g, vrhe->g) || mpz_cmp(vtmf->h, vrhe->h))
+			return false;
+	
+	if (s.size() != s2.size())
+		return false;
+	
+	// check whether the elements of the shuffled stack belong to the group
+	for (size_t i = 0; i < s2.size(); i++)
+	{
+		if (!vtmf->CheckElement(s2[i].c_1) || !vtmf->CheckElement(s2[i].c_2))
+			return false;
+	}
+	
+	std::vector<mpz_ptr> R;
+	std::vector<std::pair<mpz_ptr, mpz_ptr> > e, E;
+
+	JareckiLysyanskayaEDCF *edcf = new JareckiLysyanskayaEDCF(2, 0,
+		vtmf->p, vtmf->q, vtmf->g, vtmf->h);
+	
+	TMCG_InitializeStackEquality_Groth(e, E, s, s2); // we can use the helper
+	bool return_value = vrhe->Verify_interactive_publiccoin(e, E, edcf, in, out);
+	TMCG_ReleaseStackEquality_Groth(e, E); // we can use this helper also here
+
+	delete edcf;
+	
+	return return_value;
+}
+
+bool SchindelhauerTMCG::TMCG_VerifyStackEquality_Hoogh_noninteractive
+	(const TMCG_Stack<VTMF_Card> &s, const TMCG_Stack<VTMF_Card> &s2,
+	BarnettSmartVTMF_dlog *vtmf, HooghSchoenmakersSkoricVillegasVRHE *vrhe,
+	std::istream &in)
+{
+	// check whether the parameters of VRHE and VTMF match
+	if (mpz_cmp(vtmf->p, vrhe->p) || mpz_cmp(vtmf->q, vrhe->q) || 
+		mpz_cmp(vtmf->g, vrhe->g) || mpz_cmp(vtmf->h, vrhe->h))
+			return false;
+	
+	if (s.size() != s2.size())
+		return false;
+	
+	// check whether the elements of the shuffled stack belong to the group
+	for (size_t i = 0; i < s2.size(); i++)
+	{
+		if (!vtmf->CheckElement(s2[i].c_1) || !vtmf->CheckElement(s2[i].c_2))
+			return false;
+	}
+	
+	std::vector<mpz_ptr> R;
+	std::vector<std::pair<mpz_ptr, mpz_ptr> > e, E;
+	
+	TMCG_InitializeStackEquality_Groth(e, E, s, s2); // we can use the helper
+	bool return_value = vrhe->Verify_noninteractive(e, E, in);
+	TMCG_ReleaseStackEquality_Groth(e, E); // we can use this helper also here
 	
 	return return_value;
 }
